#include "testlib.h"
#include <vector>
#include <iostream>
#include <algorithm>

using namespace std;

#define sqr(x) (x) * (x)
#define EPS 1e-6
#define EPS2 1e-6

bool eq(double a, double b) {
    return doubleDelta(a - b, 0) <= EPS;
}
bool lq(double a, double b) {
    int sgn;
    if (a - b < -EPS) sgn = -1;
    else if (a - b > EPS) sgn = 1;
    else sgn = 0;
    return doubleDelta(a - b, 0) * sgn <= -EPS;
}
bool gq(double a, double b) {
    int sgn;
    if (a - b < -EPS) sgn = -1;
    else if (a - b > EPS) sgn = 1;
    else sgn = 0;
    return doubleDelta(a - b, 0) * sgn >= EPS;
}

struct ang {
	double a, b;
};
bool operator < (const ang & p, const ang & q) {
	if (eq(p.b, 0) && eq(q.b, 0))
		return lq(p.a, q.a);
	return lq(p.a * q.b, p.b * q.a);
}

class point {
public:
    double x, y;
    point() {}
    point(double _x, double _y) {
        x = _x;
        y = _y;
    }
};

double sq (point & a, point & b, point & c) {
	return a.x * (b.y - c.y) + b.x * (c.y - a.y) + c.x * (a.y - b.y);
}

point operator-(point a, point b) {
    return point(a.x - b.x, a.y - b.y);
}

bool operator==(point a, point b) {
    return eq(a.x, b.x) && eq(a.y, b.y);
}
bool operator<(point a, point b) {
    if (eq(a.x, b.x))
        return lq(a.y, b.y);
    return lq(a.x, b.x);
}

double dist(point a, point b) {
    return sqrt(sqr(a.x - b.x) + sqr(a.y - b.y));
}
double vm(point a, point b) {
    return a.x * b.y - b.x * a.y;
}
double vm(point a, point b, point c) {
    return vm(b - a, c - a);
}
double vm2(point a, point b, point c) {
    return vm(b - a, c - b);
}


double get_ang(point a) {
    double x = a.x, y = a.y;
    if (x > EPS && y >= -EPS)
        return atan(y/x);
    if (x > EPS && y < -EPS)
        return atan(y/x) + M_PI * 2;
    if (x < -EPS)
        return atan(y/x) + M_PI;
    if (abs(x) <= EPS && y > EPS)
        return M_PI / 2;
    if (abs(x) <= EPS && y < -EPS)
        return 3 * M_PI / 2;
    return 0;
}

int main(int argc, char * argv[]) {
    setName("Checker for circle-polygon");
    registerTestlibCmd(argc, argv);

    int n, r, cx, cy;
    n = inf.readInt();
    r = inf.readInt();
    cx = inf.readInt();
    cy = inf.readInt();

    int pn = ouf.readInt();
    if (pn < 3)
        quitf(_wa, "less than 3 points in answer");
    if (pn > 222)
        quitf(_wa, "more than 222 point in answer");

    vector<point> a;
    a.resize(pn);
    for (int i = 0; i < pn; ++i) {
        a[i].x = ouf.readDouble();
        a[i].y = ouf.readDouble();

        a[i].x -= cx;
        a[i].y -= cy;
    }
    vector<point> t = a;
    sort(t.begin(), t.end());

    for (int i = 0; i < pn; ++i) {
        if (t[i] == t[(i + 1) % pn]) {
            quitf(_wa, "two same points (%.8f, %.8f) and (%.8f, %.8f). dx = %.10f , dy = %.10f", t[i].x, t[i].y, t[(i + 1) % pn].x, t[(i + 1) % pn].y,
                                                                                                doubleDelta(t[i].x, t[(i + 1) % pn].x),
                                                                                                doubleDelta(t[i].y, t[(i + 1) % pn].y));
        }
        if (!eq(dist(t[i], point(0, 0)), r))
            quitf(_wa, "point %d are not on cirlce", i + 1);
    }

    int p1 = 0, p2 = 0;
    for (int i = 0; i < pn; ++i) {
        if (gq(vm2(a[i], a[(i + 1) % pn], a[(i + 2) % pn]), 0))
            p2 = 1;
        else if (lq(vm2(a[i], a[(i + 1) % pn], a[(i + 2) % pn]), 0))
            p1 = 1;
    }

    if (p2 != 1 || p1 + p2 == 0 || p1 + p2 == 2) {
        bool second_check = true;
        for (int i = 0; i < pn - 1; ++i) {
            double ang1 = get_ang(a[i + 1]);
            double ang2 = get_ang(a[i]);
            if (lq(ang1 - ang2, 0)) {
                second_check = false;
                break;
            }
        }
        if (!second_check)
            quitf(_wa, "not convex");
    }
    t.clear();

////////////////////////////////////////////////
    
    if (lq(vm2( a[0], a[1], a[2]), 0)) reverse(a.begin(), a.end());
    int zero_id = 0;
    for (int i = 0; i < pn; ++i) {
        if (lq(a[i].x, a[zero_id].x) || eq(a[i].x, a[zero_id].x) && lq(a[i].y, a[zero_id].y))
            zero_id = i;
    }
    point zero = a[zero_id];
    rotate(a.begin(), a.begin() + zero_id, a.end());
    a.erase(a.begin());
    --pn;

    vector<ang> b (pn);
    for (int i = 0; i < pn; ++i) {
        b[i].a = a[i].y - zero.y;
        b[i].b = a[i].x - zero.x;
        if (eq(b[i].a, 0))
            b[i].b = lq(b[i].b, 0) ? -1 : 1;
    }

    for (int i = 0; i < n; ++i) {
        point q;
        q.x = inf.readInt();
        q.y = inf.readInt();
        q.x -= cx;
        q.y -= cy;

        bool in = false;
        if (eq(q.x, zero.x) && eq(q.x, zero.x))
            in = true;
        else {
            ang my = {q.y - zero.y, q.x - zero.x};
            if (eq(my.a, 0))
                my.b = my.b < 0 ? -1 : 1;
            vector<ang>::iterator it = upper_bound(b.begin(), b.end(), my);
            if (it == b.end() && eq(my.a, b[pn - 1].a) && eq(my.b, b[pn - 1].b))
                it = b.end() - 1;
            if (it != b.end() && it != b.begin()) {
                int p1 = int(it - b.begin());
                if (lq(sq(a[p1], a[p1 - 1], q), 0) || eq(sq(a[p1], a[p1 - 1], q), 0))
                    in = true;
            }
        }
        if (!in)
            quitf(_wa, "point %d not in answer", i + 1);
    }

    quitf(_ok, "ok");

    return 0;
} 
