#include <cstdio>
#include <queue>
#include <set>
#include <cassert>
#include "testlib.h"
using namespace std;

// It's only possible to compile this interactor with patched testlib
// from the package of this problem.
#ifndef PATCHED_TESTLIB
    #error You should use patched testlib from the package of this problem to compile. Sorry =(
#endif

const int N = 25;
const int K = N * N;
const int NN = N * N;
const int MAXT = 2000000;
const int AICOUNT = 5;

struct vt
{
    int y, x;
    vt()
    {
        y = x = 0;
    }
    vt(int _y, int _x)
    {
        y = _y, x = _x;
    }
    friend vt operator +(vt a, vt b)
    {
        return vt(a.y + b.y, a.x + b.x);
    }
    friend bool operator <(vt a, vt b)
    {
        return a.y * NN + a.x < b.y * NN + b.x;
    }
    friend bool operator ==(vt a, vt b)
    {
        return a.x == b.x && a.y == b.y;
    }
    friend bool operator !=(vt a, vt b)
    {
        return a.x != b.x || a.y != b.y;
    }
};

vt Q[NN];
int lpt = 0, rpt = 0;

set<vt> S;

int n, m, k;

vt pop_back()
{
    vt res = Q[lpt++];
    assert(S.erase(res) == 1);
    if (lpt == NN)
        lpt = 0;
    return res;
}

int bytes = 0;
int turn = 0;

void push_front(vt v)
{
    if (S.find(v) != S.end())
        quitf(_wa, "turn = %d, len = %d: cell (%d %d) is twice in the snake!\n", turn, (int)S.size(), v.y, v.x); 
    Q[rpt++] = v;
    if (rpt == NN)
        rpt = 0;
    assert(S.insert(v).second);
}

void push_back(vt v)
{
    if (S.find(v) != S.end())
        quitf(_wa, "turn = %d, len = %d: cell (%d %d) is twice in the snake!\n", turn, (int)S.size(), v.y, v.x);
    if (lpt == 0)
        lpt = NN;
    Q[--lpt] = v;
    assert(S.insert(v).second);
}

vt front()
{
    if (rpt == 0)
        return Q[NN - 1];
    else
        return Q[rpt - 1];
}

vt d[256];

bool finished = false;
bool player_finished = false;

vt apple;

char buf[N + 1][N + 1];

void write_field()
{
    #ifdef WRITE_FIELD
        for (int i = 0; i < n; i++)
            for (int j = 0; j < m; j++)
                buf[i][j] = '.';
        for (int i = lpt; (i + 1) != (rpt < lpt ? rpt + NN : rpt); i++)
            buf[Q[i % NN].y - 1][Q[i % NN].x - 1] = 'X';
        buf[Q[(rpt == 0) ? NN - 1 : rpt - 1].y - 1][Q[(rpt == 0) ? NN - 1 : rpt - 1].x - 1] = 'H';
        buf[Q[lpt].y - 1][Q[lpt].x - 1] = 'T';
        if (!finished)
            buf[apple.y - 1][apple.x - 1] = 'A';
        for (int i = 0; i < n; i++)
            tout << buf[i] << endl;
    #endif
}

bool make_turn(char r)
{    
    ++turn;
    
    vt h = front();
    vt t = pop_back();
    vt q = h + d[r];
    if (q.x < 1 || q.y < 1 || q.x > m || q.y > n)
        quitf(_wa, "turn = %d, len = %d: snake's head (%d %d) is out of the field", turn, (int)S.size(), q.y, q.x);
    push_front(h + d[r]);
    
    if (S.find(apple) != S.end())
    {
        assert(apple == Q[(rpt == 0) ? NN - 1 : rpt - 1]);
        push_back(t);
        tout << format("turn = %d, len = %d: ate apple at (%d %d)\n", turn, (int)S.size(), apple.y, apple.x);
        return true;
    }
    return false;
}

random_t myrnd;

// [0] TEST

int apt = -1;

void test_init()
{
    apt = inf.readInt();
}

void test_next()
{
    while (apt)
    {
        apt--;
        int y = inf.readInt();
        int x = inf.readInt();
        if (S.find(vt(y, x)) != S.end())
            continue;
        apple = vt(y, x);
        return;
    }
    finished = true;
}

// END OF TEST

// [1] RANDOM

int max_len = 0;

void random_init()
{
    max_len = inf.readInt();
}

void random_next()
{
    if (S.size() == max_len)
        finished = true;
    else
    {
        while (true)
        {
            apple = vt(myrnd.next(1, n), myrnd.next(1, m));
            if (S.find(apple) != S.end())
                continue;
            break;
        }
    }
}

// END OF RANDOM

// [2] MOST REMOTE

int D[N + 1][N + 1];

void most_remote_init()
{
    max_len = inf.readInt();
}

void most_remote_next()
{
    if (S.size() == max_len)
    {
        finished = true; 
        return;
    }
    set<pair<int, vt> > DQ;
    vt s = Q[(rpt == 0) ? NN - 1 : rpt - 1];
    for (int i = 1; i <= n; i++)
        for (int j = 1; j <= n; j++)
            DQ.insert(make_pair(D[i][j] = (vt(i, j) != s) * (int)1e8, vt(i, j)));
    while (!DQ.empty())
    {
        s = DQ.begin()->second;
        int q = DQ.begin()->first;
        DQ.erase(DQ.begin());
        for (int i = 0; i < 4; i++)
        {
            vt ns = s + d["RULD"[i]];
            if (ns.x < 1 || ns.y < 1 || ns.x > m || ns.y > n)
                continue;
            int nq = q + 1 + (S.find(vt(ns.y, ns.x)) != S.end()) * 42;
            if (D[ns.y][ns.x] > nq)
                DQ.erase(make_pair(D[ns.y][ns.x], ns)), D[ns.y][ns.x] = nq, DQ.insert(make_pair(D[ns.y][ns.x], ns));
        }
    }
    vector<vt> rem;
    for (int i = 1; i <= n; i++)
        for (int j = 1; j <= m; j++)
            if (S.find(vt(i, j)) == S.end())
            {
                if (!rem.empty() && D[rem.back().y][rem.back().x] < D[i][j])
                    rem.clear();
                if (rem.empty() || D[rem.back().y][rem.back().x] == D[i][j])
                    rem.push_back(vt(i, j));
            }
    assert(!rem.empty());
    int r = myrnd.next(0, (int)rem.size() - 1);
    apple = rem[r];
}

// END OF MOST REMOTE

// [3] CORNERS

void corners_init()
{
    max_len = inf.readInt();    
}

void corners_next()
{
    if (S.size() == max_len)
    {
        finished = true;
        return;
    }
    for (int d = 0; d <= n + m; d++)
    {
        vector<vt> rem;
        for (int x = 1; x <= m; x++)
        {
            int y[] = {d + 2 - x, n - d - 1 + x, x - m + 1 + d, n - (x - m) - d};
            for (int t = 0; t < 4; t++)
                if (y[t] >= 1 && y[t] <= n)
                    if (S.find(vt(y[t], x)) == S.end())
                        rem.push_back(vt(y[t], x));
        }
        if (!rem.empty())
        {
            apple = rem[myrnd.next(0, (int)rem.size() - 1)];
            return;
        }
    }
    assert(false);
}

// END OF CORNERS

// [4] TOP_SIDE

void top_side_init()
{
    max_len = inf.readInt();    
}

void top_side_next()
{
    if (S.size() == max_len)
    {
        finished = true;
        return;
    }
    for (int y = 1; y <= n; y++)
    {
        vector<vt> rem;
        for (int x = 1; x <= m; x++)
        {
            if (S.find(vt(y, x)) == S.end())
                rem.push_back(vt(y, x));
        }
        if (!rem.empty())
        {
            apple = rem[myrnd.next(0, (int)rem.size() - 1)];
            return;
        }
    }
    assert(false);
}

// END OF TOP_SIDE


typedef void (*ai)();
ai AINEXT[AICOUNT] = {test_next, random_next, most_remote_next, corners_next, top_side_next};
ai AIINIT[AICOUNT] = {test_init, random_init, most_remote_init, corners_init, top_side_init};

bool next_apple()
{
    if (finished)
        assert(false);
    AINEXT[k]();
    
    if (finished)
    {
        tout << format("turn = %d, len = %d: last apple eaten\n", turn, (int)S.size());
        printf("0 0\n");
        fflush(stdout);
        return false;
    }
    
    printf("%d %d\n", apple.y, apple.x);
    fflush(stdout);
    tout << format("turn = %d, len = %d: new apple appeared at (%d %d)\n", turn, (int)S.size(), apple.y, apple.x);
    return true;
}

int main(int argc, char *argv[])
{
    d['R'] = vt(0, 1);
    d['U'] = vt(-1, 0);
    d['L'] = vt(0, -1);
    d['D'] = vt(1, 0);
    registerInteraction(argc, argv);

    n = inf.readInt();
    m = inf.readInt();
    k = inf.readInt();
    assert(0 <= k && k < AICOUNT);
    AIINIT[k]();
    
    push_front(vt(1, 1));
    push_front(vt(1, 2));
    push_front(vt(1, 3));
    
    printf("%d %d\n", n, m);
    fflush(stdout);
    assert(next_apple());
    write_field();
    while (true)
    {
        string s = ouf.readString("[RULD]*");
        tout << s << endl;
        bool made = false;
        for (int i = 0; i < s.size(); i++)
            if (make_turn(s[i]))
            {
                if (i + 1 != s.size())
                    quitf(_wa, "turn = %d, len = %d: ate apple before the sequence ends (sequence length = %d, %d operations remaining)", turn, (int)S.size(), (int)s.size(), (int)s.size() - i - 1);
                made = true;
            }
        if (!made)
            quitf(_wa, "turn = %d, len = %d: sequence of movements didn't make snake eat apple =(", turn, (int)S.size());
        if (!next_apple())
            break;
        write_field();
    }
    write_field();
    tout << format("turn = %d, len = %d: finished!\n", turn, (int)S.size());
    quitf(_ok, "turn = %d, len = %d: finished\n", turn, (int)S.size());
}
